/* I Want Candy
 * A puzzle game where a player replicates an accelerating sequence of LEDs with pushbuttons.
 * The player is rewarded with candy for the highest level they reach before messing up.
 * If the player lasts for 21 levels, they get all the candy!!
 * 
 * Created for USF Makecourse Spring 2021
 */

#include <RBD_Timer.h> //timer library
#include <StepperAK.h> //stepper motor library

#define led1 37 //defining LED pins 1-5
#define led2 39
#define led3 41
#define led4 43
#define led5 45

#define pb1 2 //defining Push Button pins 1-5
#define pb2 3
#define pb3 21
#define pb4 20
#define pb5 19

#define ulnA1 5 //defining ULN2003 Stepper Motor Controller A pins 1-4
#define ulnA2 6
#define ulnA3 7
#define ulnA4 8

#define ulnB1 10 //defining ULN2003 Stepper Motor Controller B pins 1-4
#define ulnB2 11
#define ulnB3 12
#define ulnB4 13

#define buzzer 48

//VARIABLE DECLARATIONS

int led_select = 0;                             //variable used in the random led sequence selection for loop to 
                                                //keep track of the random number generated
int led_pin = 0;                                //variable used in the random led sequence selection for loop to 
                                                //track the random led pin selected
volatile float delaytime = 8;                   //delaytime used in the stepper motor functions tab
int ledseq[10] = {0,0,0,0,0,0,0,0,0,0};         //size 5 array used to store the random led sequence for comparison
                                                //to the players button input 
volatile int pbseq[10] = {0,0,0,0,0,0,0,0,0,0}; //size 5 array used to store the players pushbutton input for comparison 
                                                //to the led sequence

volatile int c = 0;                             //counter used within pushbutton interrupts to index pbseq array and as 
                                                //condition for ending player input
int c2 = 0;                                     //counter to track how many matches between ledseq and pbseq array. determines 
                                                //if player is correct and is condition for moving to next round

int level = 0;                                  //tracks players current level. increases every time player replicates led 
                                                //sequence
int win_lose_flag = 0;                          //0 = keep playing, 1 = player has lost or won and game ends
int motor_select = 0;                           //flags which motor is to be run. levels 0-10, motor #1 (motor_select = 0). levels
                                                //11-20, motor #2 (motor_select = 1)

long int motor_run_time[] = {0, 5000, 5000, 7000,  //motor run time array for led sequence blink. gets progressively longer as 
                        9000, 11000, 13000,        //level increases.
                        15000, 17000, 19000, 
                        21000, 5000, 5000, 
                        7000, 9000, 11000, 
                        13000, 15000, 17000, 
                        19000, 21000, 60000};                      

int led_on_delay[] = {300, 300, 300, 300, 200,   //delay time array for led sequence blink on. gets progressively harder as 
                   200, 200, 100, 100, 50,       //level increases.
                   300, 300, 300, 300,
                   200, 200, 200, 100, 100,
                   50};              
                   
int led_off_delay[] = {1000, 900, 800, 700, 600, //delay time array for led sequence blink off. gets progressively harder as 
                   500, 500,400,350,300,         //level increases.
                   250, 900,800, 700,
                   650, 600, 575, 550, 450,
                   350, 250};   
                                   
int led_seq_length[] = {5, 10};                 //length of led sequence. 5 for levels 0-10. 10 for levels 11-20;
int lsl_sel = 0;                                //led sequence length selection flag. 0 for levels 0-10 and 1 for levels 11-20. 
                                                //This flag is also used to reference level tier within code

RBD::Timer timer;                               //creates a timer for the motor run timing

void setup() {
  pinMode(led1, OUTPUT);   //initializing the 5 led pins as outputs
  digitalWrite(led1, LOW); //and setting them to low to ensure they are off when the program begins
  pinMode(led2, OUTPUT);
  digitalWrite(led2, LOW);
  pinMode(led3, OUTPUT);
  digitalWrite(led3, LOW);
  pinMode(led4, OUTPUT);
  digitalWrite(led4, LOW);
  pinMode(led5, OUTPUT);
  digitalWrite(led5, LOW);
  
  pinMode(pb1, INPUT);     //initializing the 5 pushbuttons as inputs
  digitalWrite(pb1, HIGH); //and setting them to high (with an internal pull up resistor) so that the register low when pressed
  pinMode(pb2, INPUT);
  digitalWrite(pb2, HIGH); 
  pinMode(pb3, INPUT);
  digitalWrite(pb3, HIGH); 
  pinMode(pb4, INPUT);
  digitalWrite(pb4, HIGH); 
  pinMode(pb5, INPUT);
  digitalWrite(pb5, HIGH);   

  pinMode(ulnA1, OUTPUT); //initializing the ULN2003 Stepper Motor Controller A pins as outputs
  pinMode(ulnA2, OUTPUT);
  pinMode(ulnA3, OUTPUT);
  pinMode(ulnA4, OUTPUT);

  pinMode(ulnB1, OUTPUT); //initializing the ULN2003 Stepper Motor Controller B pins as outputs
  pinMode(ulnB2, OUTPUT);
  pinMode(ulnB3, OUTPUT);
  pinMode(ulnB4, OUTPUT);

  pinMode(buzzer,OUTPUT); //initializing the buzzer pin as output

  randomSeed(analogRead(A4)); //Ensures LED sequences are uniquely random each time the game is played

  attachInterrupt(digitalPinToInterrupt(pb1), pb1interrupt, FALLING); //assigning the 5 pushbutton pins (I0-I4) as interrupts 
  attachInterrupt(digitalPinToInterrupt(pb2), pb2interrupt, FALLING); //that will trigger when the signal goes from high to 
  attachInterrupt(digitalPinToInterrupt(pb3), pb3interrupt, FALLING); //low (pushbutton is pressed)
  attachInterrupt(digitalPinToInterrupt(pb4), pb4interrupt, FALLING);
  attachInterrupt(digitalPinToInterrupt(pb5), pb5interrupt, FALLING);

  Serial.begin(9600);
}

void loop(){ //main loop
  
  delay(2000); //delay of 2 seconds to give player time to get ready
  
  for(int i = 0; i < 5; i++){ //this for loop flashes all five leds five times to alert player that the game is about to start
    digitalWrite(led1, HIGH);
    digitalWrite(led2, HIGH);
    digitalWrite(led3, HIGH);
    digitalWrite(led4, HIGH);
    digitalWrite(led5, HIGH);
    tone(buzzer,1000);                
    delay(500);  
    noTone(buzzer);
    digitalWrite(led1, LOW);
    digitalWrite(led2, LOW);
    digitalWrite(led3, LOW);
    digitalWrite(led4, LOW);
    digitalWrite(led5, LOW);
    delay(200);
  }

  //delay(2000); //delay of 2 seconds to give player time to get ready

  //START OF GAME
  
  while(win_lose_flag == 0){ //main game loop. while this flag = 0, the game player is still winning and game continues. 
                             //when player loses or wins all 21 levels, flag = 1 and main game loop ends
    
    delay(3000); //3 sec delay before led sequence begins
    
    if(level < 11){       //this if / else if structure sets led sequence length select and motor select to the correct level tier
      lsl_sel = 0;
      motor_select = 0;
      }
    else if(level >= 11){
      lsl_sel = 1;
      motor_select = 1;
      }
    
    
    //LED SEQUENCE PHASE
    
    for(int i = 0; i < led_seq_length[lsl_sel]; i++){ //this for loop creates a random sequence of 5 or 10 leds
      
      led_select = random(5); //assigns a random number from 0-4 to led_select variable
      
      if (led_select == 0){ //this if / if else structure assigns the led pin associated with the current random value 
                            //of led_select to the led_pin variable
        led_pin = led1;
        }
      else if (led_select == 1){
        led_pin = led2;
        }
      else if (led_select == 2){
        led_pin = led3;
        }
      else if (led_select == 3){
        led_pin = led4;
        }
      else if (led_select == 4){
        led_pin = led5;
        }
        
      ledseq[i]=led_select; //this line assigns the loops currently selected led pin to a sequential position in the ledseq array.
                            //the for loop repeats until there are 5 or 10 randomly selected led pins in the array.
                            //this array is used later for comparison to the players input button sequence.
                           
      digitalWrite(led_pin, HIGH); //each iteration of the for loop lights the current random led
      delay(led_on_delay[level]);  //a variable delay indexed to the current level for the led light up time. delay gets 
      digitalWrite(led_pin, LOW);  //shorter as level increases and turns it off again
      delay(led_off_delay[level]); //followed by a variable delay indexed to the current level before the next led lights up. 
      }                            //delay gets shorter as level increases. loop ends after 5 or 10 leds have been selected and lit
      
  Serial.print("ledseq 0: ");      //for serial monitoring to troubleshoot. Displays the LED sequence
  Serial.println(ledseq[0]);
  Serial.print("ledseq 1: ");
  Serial.println(ledseq[1]);
  Serial.print("ledseq 2: ");
  Serial.println(ledseq[2]);
  Serial.print("ledseq 3: ");
  Serial.println(ledseq[3]);
  Serial.print("ledseq 4: ");
  Serial.println(ledseq[4]);
  Serial.print("ledseq 5: ");
  Serial.println(ledseq[5]);
  Serial.print("ledseq 6: ");
  Serial.println(ledseq[6]);
  Serial.print("ledseq 7: ");
  Serial.println(ledseq[7]);
  Serial.print("ledseq 8: ");
  Serial.println(ledseq[8]);
  Serial.print("ledseq 9: ");
  Serial.println(ledseq[9]);
  Serial.println(" ");
  
    //PLAYER INPUT PHASE

    timer.setTimeout(30000); //sets timer to 30 seconds for player to enter sequence
    timer.restart();         //starts timer
    
    while ((c < 10)&&(timer.isExpired()!= 1)){ //while loop runs until player makes 5 or 10 entries or timer runs 
                                               //out (button push interrupts advance c by 1)
      if((lsl_sel == 0)&&(c == 5)){            //if level is 0-10, this ends the player entry when 5 buttons have been pressed
        c=10;
        }
      }
  
  Serial.print("pbseq 0: ");     //serial monitoring for troubleshooting. Displays the pushbutton sequence entered
  Serial.println(pbseq[0]);
  Serial.print("pbseq 1: ");
  Serial.println(pbseq[1]);
  Serial.print("pbseq 2: ");
  Serial.println(pbseq[2]);
  Serial.print("pbseq 3: ");
  Serial.println(pbseq[3]);
  Serial.print("pbseq 4: ");
  Serial.println(pbseq[4]);
  Serial.print("pbseq 5: ");
  Serial.println(pbseq[5]);
  Serial.print("pbseq 6: ");
  Serial.println(pbseq[6]);
  Serial.print("pbseq 7: ");
  Serial.println(pbseq[7]);
  Serial.print("pbseq 8: ");
  Serial.println(pbseq[8]);
  Serial.print("pbseq 9: ");
  Serial.println(pbseq[9]);
  Serial.println(" ");
  
    //PERFORMANCE EVALUATION PHASE - DID PLAYER WIN LEVEL OR LOSE?
    
    for(int j = 0; j < led_seq_length[lsl_sel]; j++){ //this loop evaluates the players entry
      if(ledseq[j] == pbseq[j]){                      //this nested loop compares the sequence to the player's entry one 
        c2++;                                         //led / button press at a time and increases counter c2 if there is a match
        }
      }

    if((lsl_sel == 0)&&(c2 != 5)){ //level is 0-10 and player didn't match sequence
      win_lose_flag = 1;           //player loses
      tone(buzzer,10);             
      delay(200);    
      noTone(buzzer); 
      delay(250);
      tone(buzzer,10);             
      delay(200);    
      noTone(buzzer); 
      }
    else if((lsl_sel == 0)&&(c2 == 5)){ //level is 0-10 and player did match sequence
      level++;                          //go up a level and keep playing
      tone(buzzer,293);             
      delay(200);    
      noTone(buzzer); 
      tone(buzzer,329);      
      delay(200);
      noTone(buzzer);     
      tone(buzzer,349);    
      delay(200);    
      noTone(buzzer);
      }
      
    else if((lsl_sel == 1)&&(c2 != 10)){ //level is 11-20 and player didn't match sequence
      win_lose_flag = 1;                 //player loses
      tone(buzzer,10);             
      delay(200);    
      noTone(buzzer); 
      delay(250);
      tone(buzzer,10);             
      delay(200);    
      noTone(buzzer); 
      }
    else if((lsl_sel == 1)&&(c2 == 10)){ //level is 11-20 and player did match sequence
      level++;                           //go up a level and keep playing
      tone(buzzer,293);             
      delay(200);    
      noTone(buzzer); 
      tone(buzzer,329);      
      delay(200);
      noTone(buzzer);     
      tone(buzzer,349);    
      delay(200);    
      noTone(buzzer);
      }

    if(level==21){
      win_lose_flag = 1; //player has beat the game!
      for(int i=0;i<5;i++){
      tone(buzzer,261);    
      delay(200);
      noTone(buzzer); 
      // Sounds the buzzer at the frequency relative to the note D in Hz   
      tone(buzzer,293);             
      delay(200);    
      noTone(buzzer); 
      // Sounds the buzzer at the frequency relative to the note E in Hz
      tone(buzzer,329);      
      delay(200);
      noTone(buzzer);     
      // Sounds the buzzer at the frequency relative to the note F in Hz
      tone(buzzer,349);    
      delay(200);    
      noTone(buzzer); 
      // Sounds the buzzer at the frequency relative to the note G in Hz
      tone(buzzer,392);            
      delay(200);
      noTone(buzzer); 
        }
      }
      
  c=0;                   //resets button counters for next loop iteration
  c2=0;

  Serial.print("level: ");    //serial monitoring for troubleshooting. Displays the current level
  Serial.println(level);
  Serial.println(" ");
  
  }                      //end of main game loop

  //GAME OVER - CANDY DISTRIBUTION PHASE - HOW MUCH CANDY?
  
  for(int i = 0; i < 5; i++){ //this for loop flashes all five leds five times to alert player that the game is over
    digitalWrite(led1, HIGH);
    digitalWrite(led2, HIGH);
    digitalWrite(led3, HIGH);
    digitalWrite(led4, HIGH);
    digitalWrite(led5, HIGH);
    delay(500);
    digitalWrite(led1, LOW);
    digitalWrite(led2, LOW);
    digitalWrite(led3, LOW);
    digitalWrite(led4, LOW);
    digitalWrite(led5, LOW);
    delay(200);
  }


  if((motor_select == 0)&&(level != 21)){ //runs motor #1 for the level indexed time if level is 0-10
    timer.setTimeout(motor_run_time[level]);
    timer.restart();
    while(timer.isExpired()!= 1) {
      forward1();
      }
    }
  else if((motor_select == 1)&&(level != 21)){ //runs motor #2 for the level indexed time if level is 11-20
    timer.setTimeout(motor_run_time[level]);
    timer.restart();
    while(timer.isExpired()!= 1) {
      backward2();
      }
    }
  else if(level == 21){                      //player has beat the game
    timer.setTimeout(motor_run_time[level]);
    timer.restart();
    while(timer.isExpired()!= 1) {
      forward1();                           //both motors run for 60 seconds. lots of candy!
      backward2();
      }
    }
    

c=0;              //resetting variables for next game
c2=0;
win_lose_flag=0;
level=0;
motor_select = 0;
lsl_sel = 0;


while (c == 0){  //LED1 flashes every 1.5 sec until a button is pressed to restart the game
  
    delay(1000);
      
    digitalWrite(led1, HIGH);
    delay(500);
    digitalWrite(led1, LOW);
  }
c = 0;

delay(2000);
}                 //end of main loop

void pb1interrupt (){
 static unsigned long last_interrupt_time = 0; //variable to record the time when the iterrupt was last run
 unsigned long interrupt_time = millis();      //variable to compare current time to last interrupt time
 if (interrupt_time - last_interrupt_time > 150){ // If interrupts come faster than 150ms, assume it's a bounce and ignore
   pbseq[c] = 0; //sets the player button sequence tracking array pbseq at the current counter index to the button number
   c++;          //increments the counter up one. The code will stop recording buttons after 5 presses.
 }
 last_interrupt_time = interrupt_time;
}

void pb2interrupt (){
 static unsigned long last_interrupt_time = 0; //variable to record the time when the iterrupt was last run
 unsigned long interrupt_time = millis();      //variable to compare current time to last interrupt time
 if (interrupt_time - last_interrupt_time > 150){ // If interrupts come faster than 150ms, assume it's a bounce and ignore
   pbseq[c] = 1; //sets the player button sequence tracking array pbseq at the current counter index to the button number
   c++;          //increments the counter up one. The code will stop recording buttons after 5 presses.
 }
 last_interrupt_time = interrupt_time;
}

void pb3interrupt (){
 static unsigned long last_interrupt_time = 0; //variable to record the time when the iterrupt was last run
 unsigned long interrupt_time = millis();      //variable to compare current time to last interrupt time
 if (interrupt_time - last_interrupt_time > 150){ // If interrupts come faster than 150ms, assume it's a bounce and ignore
   pbseq[c] = 2; //sets the player button sequence tracking array pbseq at the current counter index to the button number
   c++;          //increments the counter up one. The code will stop recording buttons after 5 presses.
 }
 last_interrupt_time = interrupt_time;
}

void pb4interrupt (){
 static unsigned long last_interrupt_time = 0; //variable to record the time when the iterrupt was last run
 unsigned long interrupt_time = millis();      //variable to compare current time to last interrupt time
 if (interrupt_time - last_interrupt_time > 150){ // If interrupts come faster than 150ms, assume it's a bounce and ignore
   pbseq[c] = 3; //sets the player button sequence tracking array pbseq at the current counter index to the button number
   c++;          //increments the counter up one. The code will stop recording buttons after 5 presses.
 }
 last_interrupt_time = interrupt_time;
}

void pb5interrupt (){
 static unsigned long last_interrupt_time = 0; //variable to record the time when the iterrupt was last run
 unsigned long interrupt_time = millis();      //variable to compare current time to last interrupt time
 if (interrupt_time - last_interrupt_time > 150){ // If interrupts come faster than 150ms, assume it's a bounce and ignore
   pbseq[c] = 4; //sets the player button sequence tracking array pbseq at the current counter index to the button number
   c++;          //increments the counter up one. The code will stop recording buttons after 5 presses.
 }
 last_interrupt_time = interrupt_time;
}
